/*++

Copyright (c) 2004 Alexander A. Telyatnikov (Alter)

Module Name:
    PciDumpc.cpp

Abstract:
    This is simple utility that builds full list of PCI devices and
    assigned rosources (IO/Memory ranges, IRQ, etc)

Author:
    Alexander A. Telyatnikov (Alter)

Environment:
    User mode only

Notes:

    THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
    OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
    NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
    THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Revision History:

--*/

#include <windows.h>
#include <stdio.h>
#include "..\Driver\PCIDump.h"
#include "SvcManLib.h"

typedef struct _str2id {
    ULONG id;
    CCHAR* str;
} str2id;

char*
OnUpdateVendorId(
    ULONG VendorID
    ) 
{
    const str2id v_names[] = {
        {0x0000, "Gammagraphx, Inc."},
        {0x0033, "Paradyne Corp."},
        {0x003D, "Lockheed Martin Corp."},
        {0x0070, "Hauppauge Computer Works Inc."},
        {0x0675, "Dynalink"},
        {0x0700, "Lava Computer MFG Inc."},
        {0x0815, "LinTech GmbH"},
        {0x0871, "Berkom"},
        {0x0914, "Shanghai Dare Technologies Ltd."},
        {0x09C1, "Arris"},
        {0x0A89, "BREA Technologies Inc."},
        {0x0E11, "Compaq Computer Corp."},
        {0x1000, "LSI Logic"},
        {0x1002, "ATI Technologies"},
        {0x1003, "ULSI"},
        {0x1004, "VLSI Technology"},
/*        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},
        {0x1002, "ATI"},*/
        {0x1022, "AMD"},
        {0x102B, "Matrox"},
        {0x1039, "SiS"},
        {0x103C, "Hewlett-Packard Company"},
        {0x105a, "Promise"},
        {0x1065, "Texas Microsystems"},
        {0x1078, "Cyrix"},
        {0x1080, "Cypress"},
        {0x1095, "CMD"},
        {0x10b9, "Acer Labs"},
        {0x10de, "nVIDIA"},
        {0x10ec, "ATI"},
        {0x1103, "High Point"},
        {0x1106, "VIA"},
        {0x1166, "Server Works"},
        {0x16ca, "Cenatek"},
        {0x8086, "Intel"},
        {0xffff, 0}
    };
	
    for(int i=0; v_names[i].str; i++) {
        if(v_names[i].id == VendorID) {
            return v_names[i].str;
        }
    }
    return "";
}

char*
OnUpdateDeviceId(
    ULONG VendorID,
    ULONG DeviceID
    ) 
{
    const str2id v_names[] = {
        {0x808624cb, "ICH4 ATA 100"},
        {0x10b95229, "Aladdin ATA/ATA33 controller"},
        {0x10de01bc, "nForce ATA"},
        {0x10227441, "AMD 768 ATA"},
        {0x11060571, "ATAxxx controller"},
        {0x10395513, "5591 ATA33"},
        {0x10950649, "CMD 649 ATA 100"},
        {0x1080c693, "82C693 ATA"},
        {0x10780102, "5530 ATA33"},
        {0x11660211, "ROSB4 ATA33"},
        {0x105a4d68, "TX2 ATA100"},
        {0x11030004, "HPT366/368/370/372 ATA"},
        {0x16ca0001, "Rocket Drive"},
        {0x10025046, "Rage 128 Pro"},
        {0xffff, 0}
    };
    ULONG z = (VendorID << 16) | DeviceID;
	
    for(int i=0; v_names[i].str; i++) {
        if(v_names[i].id == z) {
            return v_names[i].str;
        }
    }
    return "";	
}

char*
OnUpdateClassId(
    ULONG BaseClass
    ) 
{
    const str2id v_names[] = {
        {0x01, "Mass Storage"},
        {0x02, "Network Card"},
        {0x03, "Video Adapter"},
        {0x04, "Multimedia Device"},
        {0x05, "Memory"},
        {0x06, "Bridge Device"},
        {0x07, "Simple Communication Device"},
        {0x08, "Base Peripheral"},
        {0x09, "Input Device"},
        {0x0a, "Docking Device"},
        {0x0b, "Processor"},
        {0x0c, "Serial Bus"},
        {0xffff, 0}
    };
	
    for(int i=0; v_names[i].str; i++) {
        if(v_names[i].id == BaseClass) {
            return v_names[i].str;
        }
    }
    return "";	
}

void
PrintBaseAddress(
    PCI_COMMON_CONFIG* pciData,
    ULONG i
    )
{
//    ULONG tmp;

    if(!pciData->u.type0.BaseAddresses[i])
        return;
    printf("\tBase Address %d: %08x\t%s\n", i, pciData->u.type0.BaseAddresses[i] & 0xfffffffe,
        (pciData->u.type0.BaseAddresses[i] & PCI_ADDRESS_IO_SPACE) ? "I/O" : "Mem");

/*    tmp = pciData->u.type0.BaseAddresses[i] & PCI_ADDRESS_MEMORY_TYPE_MASK; \
    SetDlgItemText(IDC_BASEADDRMT##i, \
    (tmp) ? (tmp == 2 ? PCITXT("20 bit") : PCITXT("64 bit")) : PCITXT("32 bit")); \
    pBt = (CButton*)GetDlgItem(IDC_BASEADDR_CHK_PF##i); \
    pBt->SetCheck((CurDevInfo.u.type0.BaseAddresses[i] & PCI_ADDRESS_MEMORY_PREFETCHABLE) ? (h != ((HANDLE)-1)) : 0);*/
}

VOID
PrintDevInfo(
    ULONG              BusId,
    ULONG              DevId,
    ULONG              DevFunc,
    PCI_COMMON_CONFIG* pciData
    )
{
    BOOLEAN mf;
    
    printf("\nBus %02x  Device %02x  Function %02x\n", BusId,  
                                                        DevId,  
                                                        DevFunc);
    printf("\tVendor: %04x\t%s\n", pciData->VendorID, OnUpdateVendorId(pciData->VendorID));
    printf("\tDevice: %04x\t%s\n", pciData->DeviceID, OnUpdateDeviceId(pciData->VendorID, pciData->DeviceID));
    printf("\tRevision: %02x\n", pciData->RevisionID);

    printf("\tDevice class: %02x\t%s\n", pciData->BaseClass, OnUpdateClassId(pciData->BaseClass));
    printf("\tDevice subclass: %02x\n", pciData->SubClass);
//    printf("\tDevice sub-subclass: ??\n");

    mf = (pciData->HeaderType & PCI_MULTIFUNCTION) != 0;
    if(mf)
        printf("\tMultifunction device\n");

    if((pciData->HeaderType & ~PCI_MULTIFUNCTION) == PCI_DEVICE_TYPE) {

        PrintBaseAddress(pciData, 0);
        PrintBaseAddress(pciData, 1);
        PrintBaseAddress(pciData, 2);
        PrintBaseAddress(pciData, 3);
        PrintBaseAddress(pciData, 4);
        PrintBaseAddress(pciData, 5);

        if(pciData->u.type0.ROMBaseAddress) {
            printf("\tROM base: %08x\t%s\n", pciData->u.type0.ROMBaseAddress,
                (pciData->u.type0.ROMBaseAddress & PCI_ROMADDRESS_ENABLED) ? "Enabled" : "Disabled");
        }

        printf("\tInterrupt line: %02x\tInterrupt pin: %02x\tMin_Gnt: %02x\tMaxLat: %02x\n",
            pciData->u.type0.InterruptLine,
            pciData->u.type0.InterruptPin,
            pciData->u.type0.MinimumGrant,
            pciData->u.type0.MaximumLatency);
        printf("\tCache line size: %02x\tLatency timer: %02x\tHeader type: %02x\tBIST: %02x\n",
            pciData->CacheLineSize,
            pciData->LatencyTimer,
            pciData->HeaderType,
            pciData->BIST);
        printf("\tI/O:%d\tMem:%d\tBusMast:%d\tSpecial:%d\tMemInv:%d\n",
            pciData->Command & PCI_ENABLE_IO_SPACE     ? 1 : 0,
            pciData->Command & PCI_ENABLE_MEMORY_SPACE ? 1 : 0,
            pciData->Command & PCI_ENABLE_BUS_MASTER   ? 1 : 0,
            pciData->Command & PCI_ENABLE_SPECIAL_CYCLES ? 1 : 0,
            pciData->Command & PCI_ENABLE_WRITE_AND_INVALIDATE ? 1 : 0);

    }

}

VOID
BuildDevList()
{
    ULONG BusId=0;
    ULONG DevId=0;
    ULONG DevFunc=0;

    ULONG returned;

    PciDump_GetHeader_USER_IN  buf_in;
    PciDump_GetHeader_USER_OUT buf_out;

    HANDLE h;

    h = CreateFile(NT_PciDump_USER_NAME, GENERIC_READ | GENERIC_WRITE,
                   FILE_SHARE_READ | FILE_SHARE_WRITE,
                   NULL,
                   OPEN_EXISTING,
                   FILE_ATTRIBUTE_NORMAL/* | FILE_FLAG_NO_BUFFERING*/,  NULL);

    while(h != ((HANDLE)-1)) {

        buf_in.BusNumber = BusId;
        buf_in.DeviceNumber = 0;
        buf_in.FunctionNumber = 0;
        
        if(!DeviceIoControl(h,IOCTL_PciDump_GetHeader,
                            &buf_in,sizeof(buf_in),
                            &buf_out,sizeof(buf_out),
                            &returned,NULL)
                  ||
            buf_out.status == 0) {
            CloseHandle(h);
            h = ((HANDLE)-1);
            break;
        }
        
        for(DevId=0; DevId<PCI_MAX_DEVICES; DevId++) {
            
            buf_in.BusNumber = BusId;
            buf_in.DeviceNumber = DevId;
            buf_in.FunctionNumber = 0;

            DeviceIoControl(h,IOCTL_PciDump_GetHeader,
                                &buf_in,sizeof(buf_in),
                                &buf_out,sizeof(buf_out),
                                &returned,NULL);

            if(buf_out.status == 2)
                continue;

/*            buf_in.BusNumber = BusId;
            buf_in.DeviceNumber = DevId;
            buf_in.FunctionNumber = 1;

            DeviceIoControl(h,IOCTL_PciDump_GetHeader,
                                &buf_in,sizeof(buf_in),
                                &buf_out,sizeof(buf_out),
                                &returned,NULL);

            if(buf_out.status == 2)
                continue;*/

            for(DevFunc=0; DevFunc<PCI_MAX_FUNCTION; DevFunc++) {

                buf_in.BusNumber = BusId;
                buf_in.DeviceNumber = DevId;
                buf_in.FunctionNumber = DevFunc;

                DeviceIoControl(h,IOCTL_PciDump_GetHeader,
                                    &buf_in,sizeof(buf_in),
                                    &buf_out,sizeof(buf_out),
                                    &returned,NULL);

                if(buf_out.status == 2)
                    continue;

                // Dump PCI device
                PrintDevInfo(BusId, DevId, DevFunc, &buf_out.pciData);
            }
        }

        BusId++;
    }
      
    if(h != ((HANDLE)-1)) {
        CloseHandle(h);
        h = ((HANDLE)-1);
    }
}

void main (int argc, char* argv[]) {
//    HANDLE h;
    LONG l=0;
    PUCHAR buff=NULL;
    PUCHAR buff_j=NULL;
    PUCHAR buff_i=NULL;
//    LONG tmp;
    LONG i=0, j=0;
    ULONG k;
    
    k = NtServiceIsRunning(NT_PciDump_SVC_NAME);
    if(k == 1) {
        // Ok
    } else
    if(k == 0) {
        k = NtServiceStart(NT_PciDump_SVC_NAME);
    }
    if(k >= 0) {
        BuildDevList();
    }

    exit(0);
}
